DROP TABLE IF EXISTS `slice_engines`;
CREATE TABLE `slice_engines` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `engine_name` varchar(255) NOT NULL,
  `engine_path` varchar(255) NOT NULL,
  `engine_description` text NOT NULL,
  `is_featured` tinyint(1)  NOT NULL,
  `is_public` tinyint(1) NOT NULL,
  `add_date` datetime NOT NULL,
  `default_config_id` int(11) unsigned NOT NULL,
  PRIMARY KEY (`id`),
  KEY `engine_name` (`engine_name`),
  KEY `is_featured` (`is_featured`),
  KEY `is_public` (`is_public`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

DROP TABLE IF EXISTS `slice_configs`;
CREATE TABLE `slice_configs` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `fork_id` int(11) unsigned NOT NULL,
  `engine_id` int(11) unsigned NOT NULL,
  `user_id` int(11) unsigned NOT NULL,
  `config_name` varchar(255) NOT NULL,
  `config_data` text NOT NULL,
  `add_date` datetime NOT NULL,
  `edit_date` datetime NOT NULL,
  PRIMARY KEY (`id`),
  KEY `fork_id` (`fork_id`),
  KEY `user_id` (`user_id`),
  KEY `engine_id` (`engine_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

DROP TABLE IF EXISTS `slice_jobs`;
CREATE TABLE `slice_jobs` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `user_id` int(11) unsigned NOT NULL,
  `job_id` int(11) unsigned NOT NULL,
  `input_id` int(11) unsigned NOT NULL,
  `output_id` int(11) unsigned NOT NULL,
  `output_log` text NOT NULL,
  `slice_config_id` int(11) unsigned NOT NULL,
  `slice_config_snapshot` text NOT NULL,
  `worker_token` char(40) NOT NULL,
  `worker_name` varchar(255) NOT NULL,
  `status` enum('available', 'slicing', 'complete', 'failure'),
  `progress` float NOT NULL DEFAULT '0',
  `add_date` datetime NOT NULL,
  `taken_date` datetime NOT NULL,
  `finish_date` datetime NOT NULL,
  PRIMARY KEY (`id`),
  KEY `user_id` (`user_id`),
  KEY `job_id` (`job_id`),
  KEY `slice_config_id` (`slice_config_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

ALTER TABLE bots add `slice_config_id` int(11) unsigned NOT NULL;
ALTER TABLE bots add key `slice_config_id` (`slice_config_id`);
ALTER TABLE bots add `slice_engine_id` int(11) unsigned NOT NULL;
ALTER TABLE bots add key `slice_engine_id` (`slice_engine_id`);
alter table bots modify status enum('idle','slicing','working','waiting','error','maintenance','offline') default 'idle';

alter table jobs modify status enum('available','taken','slicing','downloading','qa','complete','failure') not null default 'available';
alter table jobs add slice_complete_time datetime not null after finished_time;
alter table jobs add source_file_id int(11) unsigned not null default 0 after queue_id;
alter table jobs add slice_job_id int(11) unsigned not null default 0 after file_id;

alter table slice_jobs drop worker_token;
alter table slice_jobs drop worker_name;
alter table slice_jobs add `uid` char(40) NOT NULL;
alter table slice_jobs add error_log text after output_log;
alter table slice_jobs modify status enum('available','slicing','pending','complete','failure','expired') default 'available';

alter table s3_files add source_url text;

INSERT INTO `slice_engines` (`id`, `engine_name`, `engine_path`, `engine_description`, `is_featured`, `is_public`, `add_date`, `default_config_id`)
VALUES
	(1,'Slic3r 0.9.8','slic3r-0.9.8','An awesome slicing engine that is compatible with RepRap, Ultimaker, RepRap, and more!',0,1, now(),1);

INSERT INTO `slice_configs` (`id`, `fork_id`, `engine_id`, `user_id`, `config_name`, `config_data`, `add_date`, `edit_date`)
VALUES
(1,0,1,1,'Default','# generated by Slic3r 0.9.8 on Wed Feb  6 12:25:51 2013\nbed_size = 200,200\nbed_temperature = 0\nbottom_solid_layers = 3\nbridge_fan_speed = 100\nbridge_flow_ratio = 1\nbridge_speed = 60\nbrim_width = 0\ncomplete_objects = 0\ncooling = 1\ndefault_acceleration = 0\ndisable_fan_first_layers = 1\nduplicate = 1\nduplicate_distance = 6\nduplicate_grid = 1,1\nend_gcode = M104 S0 ; turn off temperature\\nG28 X0  ; home X axis\\nM84     ; disable motors\nexternal_perimeter_speed = 100%\nextra_perimeters = 1\nextruder_clearance_height = 20\nextruder_clearance_radius = 20\nextruder_offset = 0x0\nextrusion_axis = E\nextrusion_multiplier = 1\nextrusion_width = 0\nfan_always_on = 0\nfan_below_layer_time = 60\nfilament_diameter = 3\nfill_angle = 45\nfill_density = 0.4\nfill_pattern = rectilinear\nfirst_layer_bed_temperature = 0\nfirst_layer_extrusion_width = 200%\nfirst_layer_height = 100%\nfirst_layer_speed = 30%\nfirst_layer_temperature = 200\ng0 = 0\ngap_fill_speed = 20\ngcode_arcs = 0\ngcode_comments = 0\ngcode_flavor = reprap\ninfill_acceleration = 0\ninfill_every_layers = 1\ninfill_extruder = 1\ninfill_extrusion_width = 0\ninfill_speed = 60\nlayer_gcode = \nlayer_height = 0.4\nmax_fan_speed = 100\nmin_fan_speed = 35\nmin_print_speed = 10\nmin_skirt_length = 0\nnotes = \nnozzle_diameter = 0.5\nonly_retract_when_crossing_perimeters = 0\noutput_filename_format = [input_filename_base].gcode\nperimeter_acceleration = 0\nperimeter_extruder = 1\nperimeter_extrusion_width = 0\nperimeter_speed = 30\nperimeters = 3\npost_process = \nprint_center = 100,100\nrandomize_start = 1\nretract_before_travel = 2\nretract_length = 1\nretract_length_toolchange = 10\nretract_lift = 0\nretract_restart_extra = 0\nretract_restart_extra_toolchange = 0\nretract_speed = 30\nrotate = 0\nscale = 1\nskirt_distance = 6\nskirt_height = 1\nskirts = 1\nslowdown_below_layer_time = 15\nsmall_perimeter_speed = 30\nsolid_fill_pattern = rectilinear\nsolid_infill_below_area = 70\nsolid_infill_every_layers = 0\nsolid_infill_speed = 60\nstart_gcode = G28 ; home all axes\nsupport_material = 0\nsupport_material_angle = 0\nsupport_material_extruder = 1\nsupport_material_extrusion_width = 0\nsupport_material_pattern = rectilinear\nsupport_material_spacing = 2.5\nsupport_material_speed = 60\nsupport_material_threshold = 0\ntemperature = 200\nthreads = 2\ntoolchange_gcode = \ntop_solid_infill_speed = 50\ntop_solid_layers = 3\ntravel_speed = 130\nuse_relative_e_distances = 0\nvibration_limit = 0\nz_offset = 0\n','2013-02-06 04:25:51','2013-02-06 04:25:51');
